"""Sphinx configuration."""

import os
import sys

import kafkaconnect
import lsst_sphinx_bootstrap_theme

# Work around Sphinx bug related to large and highly-nested source files
sys.setrecursionlimit(2000)

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.napoleon",
    "sphinx_click.ext",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx.ext.ifconfig",
    "sphinx_automodapi.automodapi",
    "sphinx_automodapi.smart_resolver",
    "documenteer.sphinxext",
]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
# source_suffix = ['.rst', '.md']
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "kafka-connect-manager"
copyright = "2020" "Association of Universities for Research in Astronomy"
author = "LSST SQuaRE"

version = kafkaconnect.__version__
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ["_build", "README.rst"]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# The reST default role cross-links Python (used for this markup: `text`)
default_role = "py:obj"

# Intersphinx

intersphinx_mapping = {
    # 'python': ('https://docs.python.org/3/', None),
}

# -- Options for linkcheck builder ----------------------------------------

linkcheck_retries = 2

# -- Options for HTML output ----------------------------------------------

templates_path = [
    "_templates",
    lsst_sphinx_bootstrap_theme.get_html_templates_path(),
]

html_theme = "lsst_sphinx_bootstrap_theme"
html_theme_path = [lsst_sphinx_bootstrap_theme.get_html_theme_path()]


html_context = {
    # Enable "Edit in GitHub" link
    "display_github": True,
    # https://{{ github_host|default("github.com") }}/{{ github_user }}/
    #     {{ github_repo }}/blob/
    #     {{ github_version }}{{ conf_py_path }}{{ pagename }}{{ suffix }}
    "github_user": "lsst-sqre",
    "github_repo": "kafka-connect-manager",
    "conf_py_path": "docs/",
    # GITHUB_REF is available in GitHub Actions, but master is a safe default
    "github_version": os.getenv("GITHUB_REF", default="master") + "/",
}

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {"logotext": project}

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
html_short_title = f"{project}"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# Do not copy reST source for each page into the build
html_copy_source = False

# If false, no module index is generated.
html_domain_indices = True

# If false, no index is generated.
html_use_index = True

# API Reference ==============================================================

napoleon_google_docstring = False
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = False
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_keyword = True  # TODO
napoleon_use_param = True
napoleon_use_rtype = True

autosummary_generate = True

automodapi_inheritance_diagram = True
automodapi_toctreedirnm = "api"
automodsumm_inherited_members = True

# Docstrings for classes and methods are inherited from parents.
autodoc_inherit_docstrings = True

# Class documentation should only contain the class docstring and
# ignore the __init__ docstring, account to LSST coding standards.
autoclass_content = "class"

# Default flags for automodapi directives. Special members are dunder
# methods.
autodoc_default_options = {
    "show-inheritance": True,
    "special-members": True,
}

# Render inheritance diagrams in SVG
graphviz_output_format = "svg"

graphviz_dot_args = [
    "-Nfontsize=10",
    "-Nfontname=Helvetica Neue, Helvetica, Arial, sans-serif",
    "-Efontsize=10",
    "-Efontname=Helvetica Neue, Helvetica, Arial, sans-serif",
    "-Gfontsize=10",
    "-Gfontname=Helvetica Neue, Helvetica, Arial, sans-serif",
]

# TODO extension =============================================================

todo_include_todos = False



"""CLI to create the S3 Sink connector.

See https://docs.confluent.io/current/connect/kafka-connect-s3
"""

__all__ = ["create_s3_sink"]

import json
import time
from typing import List

import click

from kafkaconnect.connect import Connect
from kafkaconnect.s3_sink.config import S3Config
from kafkaconnect.topics import Topic


@click.command("s3-sink")
@click.argument("topiclist", nargs=-1, required=False)
@click.option(
    "-n",
    "--name",
    "name",
    envvar="KAFKA_CONNECT_NAME",
    default="s3-sink",
    show_default=True,
    help=(
        "Name of the connector. Alternatively set via the "
        "$KAFKA_CONNECT_NAME env var."
    ),
)
@click.option(
    "-b",
    "--bucket-name",
    "s3_bucket_name",
    envvar="KAFKA_CONNECT_S3_BUCKET_NAME",
    default="mybucket",
    show_default=True,
    help=(
        "S3 bucket name. Must exist already. Alternatively set via the "
        "$KAFKA_CONNECT_S3_BUCKECT_NAME env var."
    ),
)
@click.option(
    "-r",
    "--region",
    "s3_region",
    envvar="KAFKA_CONNECT_S3_REGION",
    default="us-east-1",
    show_default=True,
    help=(
        "S3 region.Alternatively set via the $KAFKA_CONNECT_S3_REGION env var."
    ),
)
@click.option(
    "-id",
    "--aws-access-key-id",
    "aws_access_key_id",
    envvar="AWS_ACCESS_KEY_ID",
    default="",
    show_default=True,
    help=(
        "The AWS access key ID used to authenticate personal AWS credentials."
    ),
)
@click.option(
    "-secret",
    "--aws-secret-access-key",
    "aws_secret_access_key",
    envvar="AWS_SECRET_ACCESS_KEY",
    default="",
    show_default=True,
    help=(
        "The secret access key used to authenticate personal AWS credentials."
    ),
)
@click.option(
    "-d",
    "--topics-dir",
    "topics_dir",
    envvar="KAFKA_CONNECT_S3_TOPICS_DIR",
    default="topics",
    show_default=True,
    help=(
        "Top level directory to store the data ingested from Kafka. "
        "Alternatively set via the $KAFKA_CONNECT_S3_TOPICS_DIR env var."
    ),
)
@click.option(
    "-s",
    "--s3-schema-compatibility",
    "s3_schema_compatibility",
    envvar="KAFKA_CONNECT_S3_SCHEMA_COMPATIBILITY",
    default="NONE",
    show_default=True,
    help=(
        "The S3 schema compatibility mode. "
        "The supported configurations are NONE, BACKWARD, FORWARD and FULL."
    ),
)
@click.option(
    "--flush-size",
    "flush_size",
    envvar="KAFKA_CONNECT_S3_FLUSH_SIZE",
    default="3600",
    show_default=True,
    help=(
        "Number of records written to store before invoking file commits. "
        "By default this is set to six times the number of records expected "
        "for an output stream of 1Hz within the default rotate_interval_ms "
        "value. This way the rotate_interval_ms configuration takes "
        "precedence over the flush_size configuration. But flush_size "
        "still works as a maximum limit to invoke file commits when the "
        "connect-s3-sink consumer accumulates 3600 records. "
        "Alternatively set via the $KAFKA_CONNECT_S3_FLUSH_SIZE env var."
    ),
)
@click.option(
    "--rotate-interval-ms",
    "rotate_interval_ms",
    envvar="KAFKA_CONNECT_S3_ROTATE_INTERVAL_MS",
    default="600000",
    show_default=True,
    help=(
        "The time interval in milliseconds to invoke file commits. "
        "Use this option to control the size of the objects in S3. "
        "For example, if the output data stream is 1Hz, a rotate "
        "interval of 600 seconds will create a file with aproximatelly"
        "600 records if less than flush_size. Note that the lag of the"
        "connect-s3-sink consumer will increase until it accumulates "
        "records within 600 seconds and it will decrease again after "
        "the file is commited. For an hourly partitioner this configuration "
        "should create 6 parquet files at the destination path. "
        "Alternatively set via the $KAFKA_CONNECT_S3_ROTATE_INTERVAL_MS "
        "env var."
    ),
)
@click.option(
    "-p",
    "--partition-duration-ms",
    "partition_duration_ms",
    envvar="KAFKA_CONNECT_S3_PARTITION_DURATION_MS",
    default="3600000",
    show_default=True,
    help=(
        "The duration of a partition in milliseconds used by "
        "``TimeBasedPartitioner``. Alternatively set via the "
        "$KAFKA_CONNECT_S3_PARTITION_DURATION_MS env var."
    ),
)
@click.option(
    "--path-format",
    "path_format",
    envvar="KAFKA_CONNECT_S3_PATH_FORMAT",
    default="'year'=YYYY/'month'=MM/'day'=dd/'hour'=HH",
    show_default=True,
    help=(
        "Pattern used to format the path in the S3 object name. Portion of "
        " the path generated by the S3 connector’s partitioner. The default "
        " is for an hourly partitioner. Alternatively set via the "
        "$KAFKA_CONNECT_S3_PATH_FORMAT env var."
    ),
)
@click.option(
    "--topic-regex",
    "topic_regex",
    envvar="KAFKA_CONNECT_TOPIC_REGEX",
    default=".*",
    show_default=True,
    help=(
        "Regex for selecting topics. Alternatively set via the "
        "$KAFKA_CONNECT_TOPIC_REGEX env var."
    ),
)
@click.option(
    "--dry-run",
    is_flag=True,
    help=(
        "Show the InfluxDB Sink Connector configuration but do not create "
        "the connector."
    ),
)
@click.option(
    "--auto-update",
    is_flag=True,
    help=(
        "Check for new topics and update the connector. See also the "
        "``--check-interval`` option."
    ),
)
@click.option(
    "-v",
    "--validate",
    is_flag=True,
    help="Validate the connector configuration before creating.",
)
@click.option(
    "-c",
    "--check-interval",
    "check_interval",
    envvar="KAFKA_CONNECT_CHECK_INTERVAL",
    default="15000",
    show_default=True,
    help=(
        "The interval, in milliseconds, to check for new topics and update"
        "the connector. Alternatively set via the "
        "$KAFKA_CONNECT_CHECK_INTERVAL env var."
    ),
)
@click.option(
    "-e",
    "--excluded-topics",
    "excluded_topics",
    envvar="KAFKA_CONNECT_EXCLUDED_TOPICS",
    default="",
    show_default=True,
    help=(
        "Comma separated list of topics to exclude from "
        "selection. Alternatively set via the "
        "$KAFKA_CONNECT_EXCLUDED_TOPICS env var."
    ),
)
@click.option(
    "--locale",
    "locale",
    envvar="KAFKA_CONNECT_S3_LOCALE",
    default="en-US",
    show_default=True,
    help=(
        "The locale to use when partitioning with `TimeBasedPartitioner`. "
        "Alternatively set via the $KAFKA_CONNECT_S3_LOCALE env var."
    ),
)
@click.option(
    "--timezone",
    "timezone",
    envvar="KAFKA_CONNECT_S3_TIMEZONE",
    default="UTC",
    show_default=True,
    help=(
        "The timezone to use when partitioning with `TimeBasedPartitioner`."
        "Alternatively set via the $KAFKA_CONNECT_S3_TIMEZONE"
    ),
)
@click.option(
    "--timestamp-extractor",
    "timestamp_extractor",
    envvar="KAFKA_CONNECT_TIMESTAMP_EXTRACTOR",
    default="Record",
    show_default=True,
    help=(
        "The extractor determines how to obtain a timestamp from each record. "
        "Values can be Wallclock to use the system time when "
        "the record is processed, Record to use the timestamp of the "
        "Kafka record denoting when it was produced or stored by the broker, "
        "RecordField to extract the timestamp from one of the fields in the "
        "record’s value as specified by the timestamp_field configuration "
        "property. Alternatively set via the "
        "$KAFKA_CONNECT_TIMESTAMP_EXTRACTOR env var."
    ),
)
@click.option(
    "--timestamp-field",
    "timestamp_field",
    envvar="KAFKA_CONNECT_TIMESTAMP_FIELD",
    default="time",
    show_default=True,
    help=(
        "The record field to be used as timestamp by the timestamp extractor. "
        "Only applies if timestamp_extractor is set to `RecordField`. "
        "Alternatively set via the $KAFKA_CONNECT_TIMESTAMP_FIELD env var."
    ),
)
@click.option(
    "-t",
    "--tasks-max",
    "tasks_max",
    envvar="KAFKA_CONNECT_TASKS_MAX",
    default="1",
    show_default=True,
    help=(
        "Number of Kafka Connect tasks. Alternatively set via the "
        "$KAFKA_CONNECT_TASKS_MAX env var."
    ),
)
@click.pass_context
def create_s3_sink(
    ctx: click.Context,
    topiclist: tuple,
    name: str,
    s3_bucket_name: str,
    s3_region: str,
    aws_access_key_id: str,
    aws_secret_access_key: str,
    topics_dir: str,
    s3_schema_compatibility: str,
    flush_size: str,
    rotate_interval_ms: str,
    partition_duration_ms: str,
    path_format: str,
    topic_regex: str,
    dry_run: bool,
    auto_update: bool,
    validate: bool,
    check_interval: str,
    excluded_topics: str,
    locale: str,
    timezone: str,
    timestamp_extractor: str,
    timestamp_field: str,
    tasks_max: str,
) -> int:
    """Create an instance of the S3 Sink connector.

    A list of topics can be specified using the TOPICLIST argument.
    If not, topics are discovered from Kafka. Use the ``--topic-regex`` and
    ``--excluded_topics`` options to help in selecting the topics
    that you want to write to S3. To check for new topics and update
    the connector configuration use the
    ``--auto-update`` and ``--check-interval`` options.
    """
    # Get configuration from the main command
    if ctx.parent:
        config = ctx.parent.obj["config"]
    # Connector configuration
    s3config = S3Config(
        name=name,
        s3_bucket_name=s3_bucket_name,
        s3_region=s3_region,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        topics_dir=topics_dir,
        s3_schema_compatibility=s3_schema_compatibility,
        flush_size=int(flush_size),
        rotate_interval_ms=int(rotate_interval_ms),
        partition_duration_ms=int(partition_duration_ms),
        path_format=path_format,
        locale=locale,
        timezone=timezone,
        timestamp_extractor=timestamp_extractor,
        timestamp_field=timestamp_field,
        tasks_max=int(tasks_max),
    )
    # The variadic argument is a tuple
    topics: List[str] = list(topiclist)
    if not topics:
        click.echo("Discoverying Kafka topics...")
        topics = Topic(config.broker_url, topic_regex, excluded_topics).names
        n = 0 if not topics else len(topics)
        click.echo(f"Found {n} topics.")
    connect = Connect(connect_url=config.connect_url)
    if topics:
        s3config.update_topics(topics)
        # --validate option
        if validate:
            click.echo(
                connect.validate(
                    name=s3config.connector_class,
                    connect_config=s3config.asjson(),
                )
            )
            return 0
        # --dry-run option returns the connector configuration
        if dry_run:
            click.echo(s3config.asjson())
            return 0
        # Validate configuration before creating the connector
        validation = connect.validate(
            name=s3config.connector_class, connect_config=s3config.asjson(),
        )
        try:
            error_count = json.loads(validation)["error_count"]
            click.echo(f"Validation returned {error_count} error(s).")
            if error_count > 0:
                click.echo(
                    "Use the ``--validate`` option to return the validation "
                    "results."
                )
                return 0
        except Exception:
            click.echo(validation)
            return 1
        click.echo(f"Uploading {name} connector configuration...")
        connect.create_or_update(name=name, connect_config=s3config.asjson())
    if auto_update:
        while True:
            time.sleep(int(check_interval) / 1000)
            try:
                # Current list of topics from Kafka
                current_topics = Topic(
                    config.broker_url, topic_regex, excluded_topics
                ).names
                new_topics = list(set(current_topics) - set(topics))
                if new_topics:
                    click.echo("Found new topics, updating the connector...")
                    s3config.update_topics(current_topics)
                    connect.create_or_update(
                        name=name, connect_config=s3config.asjson()
                    )
                    topics = current_topics
            except KeyboardInterrupt:
                raise click.ClickException("Interruped.")
    return 0



"""Select Kafka topics."""

__all__ = ["Topic"]

import logging
import re
from typing import List, Optional, Set

from confluent_kafka import KafkaException
from confluent_kafka.admin import AdminClient

logger = logging.getLogger("kafkaconnect")


class Topic:
    """Select kafka topics.

    Parameters
    ----------
    broker_url : `str`
        The Kafka Broker URL.
    topic_regex : `str`
        Regex to select topics from Kafka.
    excluded_topics : `str`
        List of 'problematic' topics to exclude from selection.
    """

    def __init__(
        self, broker_url: str, topic_regex: str, excluded_topics: str
    ) -> None:
        self._names: Set[str] = set()
        self._admin_client = self._get_admin_client(broker_url)
        self._topic_regex = topic_regex
        self._excluded_topics = set(
            excluded_topics.replace(" ", "").split(",")
        )

    def _get_admin_client(self, broker_url: str) -> Optional[AdminClient]:
        """Get an instance of the Kafka admin client.

        Parameters
        ----------
        broker_url : `str`

        Returns
        -------
        admin_client : `AdminClient` or None
        """
        try:
            conf = {"bootstrap.servers": broker_url}
            admin_client = AdminClient(conf).list_topics(timeout=10)
        except KafkaException:
            message = (
                f"Failed to establish connection with broker " f"{broker_url}."
            )
            logger.error(message)
            admin_client = None

        return admin_client

    @property
    def names(self) -> List[str]:
        """Return a list of topic names from Kafka.

        Returns
        -------
        topics: list
            A list of topic names from Kafka. Return an empty list if no topics
            are found.
        """
        if self._admin_client:
            self._names = set(self._admin_client.topics.keys())

        if self._topic_regex:
            pattern = re.compile(self._topic_regex)
            self._names = {name for name in self._names if pattern.match(name)}

        if self._excluded_topics:
            self._names = self._names - self._excluded_topics

        return list(self._names)



"""S3 Sink connector configuration.

See https://docs.confluent.io/current/connect/kafka-connect-s3.
"""

from dataclasses import dataclass
from typing import List

from kafkaconnect.config import ConnectorConfig


@dataclass
class S3Config(ConnectorConfig):
    """S3 Sink connector configuration."""

    name: str
    """Name of the connector. Configurable."""

    s3_bucket_name: str
    """The S3 Bucket."""

    s3_region: str
    """The AWS region to be used the connector."""

    aws_access_key_id: str
    """The AWS access key ID used to authenticate personal AWS credentials."""

    aws_secret_access_key: str
    """The secret access key used to authenticate personal AWS credentials."""

    topics_dir: str
    """Top level directory to store the data ingested from Kafka."""

    s3_schema_compatibility: str
    """The S3 schema compatibility mode."""

    flush_size: int
    """Number of records written to store before invoking file commits."""

    rotate_interval_ms: int
    """The time interval in milliseconds to invoke file commits."""

    partition_duration_ms: int
    """The duration of a partition in ms, used by the TimeBasedPartitioner."""

    path_format: str
    """Pattern used to format the path in the S3 object name."""

    tasks_max: int
    """Number of Kafka Connect tasks."""

    locale: str
    """The locale to use when partitioning with TimeBasedPartitioner."""

    timezone: str
    """The timezone to use when partitioning with TimeBasedPartitioner."""

    timestamp_extractor: str
    """The extractor determines how to obtain a timestamp from each record."""

    timestamp_field: str
    """The record field to be used as timestamp by the timestamp extractor."""

    # Attributes with defaults are not configured via click
    topics: str = ""
    """Comma separated list of Kafka topics to read from."""

    connector_class: str = "io.confluent.connect.s3.S3SinkConnector"
    """S3 Sink connector class"""

    format_class: str = "io.confluent.connect.s3.format.parquet.ParquetFormat"
    """The format class to use when writing data to the store."""

    parquet_codec: str = "snappy"
    """The Parquet compression codec to be used for output files."""

    storage_class: str = "io.confluent.connect.s3.storage.S3Storage"
    """The underlying storage layer."""

    partitioner_class: str = (
        "io.confluent.connect.storage.partitioner.TimeBasedPartitioner"
    )
    """The partitioner to use when writing data to the store."""

    def update_topics(self, topics: List[str]) -> None:
        """Update the list of Kafka topics and Influx KCQL queries.

        Parameters
        ----------
        topics : `list`
            List of kafka topics.

        timestamp : `str`
            Timestamp used as influxDB time.
        """
        # Ensure uniqueness and sort topic names
        topics = list(set(topics))
        topics.sort()
        self.topics = ",".join(topics)



"""Kafkaconnect and connector configuration."""

__all__ = ["Config", "ConnectorConfig"]

import abc
import json
from dataclasses import asdict, dataclass
from typing import Any, Dict, List, Tuple


@dataclass
class Config:
    """Application configuration."""

    broker_url: str
    """The Kafka Broker URL.
    """

    connect_url: str
    """The Kafka Connect URL.

    The Kafka Connect REST API is used to manage connectors.
    """


class ConnectorConfig(metaclass=abc.ABCMeta):
    """Connector configuration interface."""

    def __subclasshook__(cls, subclass: Any) -> bool:
        """Make sure the abstract method is overriden in the subclass."""
        return (
            hasattr(subclass, "update_topics")
            and callable(subclass.update_topics)
            or NotImplemented
        )

    @abc.abstractmethod
    def update_topics(self, topics: List[str]) -> None:
        """update_topics() abstract method."""
        raise NotImplementedError

    @staticmethod
    def format_field_names(fields: List[Tuple[str, Any]]) -> Dict[str, str]:
        """Rename a field name by replacing '_' with '.'.

        Dictionary factory used with the dataclasses.asdict() method.
        """
        result = []
        for f in fields:
            name, value = f
            name = name.replace("_", ".")
            result.append((name, value))
        return dict(result)

    def asjson(self) -> str:
        """Convert dataclass instance into JSON."""
        config = asdict(self, dict_factory=self.format_field_names)
        return json.dumps(config, indent=4, sort_keys=True)



"""kafkaconnect integration tests."""

import time
from typing import Any

import pytest
import requests
from confluent_kafka import KafkaException
from confluent_kafka.admin import AdminClient, NewTopic
from requests.exceptions import ConnectionError, HTTPError

from kafkaconnect.config import Config
from kafkaconnect.connect import Connect
from kafkaconnect.influxdb_sink.config import InfluxConfig
from kafkaconnect.topics import Topic

Fixture = Any


def is_broker_responsive() -> bool:
    """Check if broker is ready."""
    try:
        admin_client = AdminClient({"bootstrap.servers": Config.broker_url})
        admin_client.list_topics(timeout=10)
    except KafkaException:
        return False
    return True


def is_connect_responsive() -> bool:
    """Check if the kafka connet API is ready."""
    try:
        uri = f"{Config.connect_url}/connectors"
        response = requests.get(uri)
        response.raise_for_status()
    except ConnectionError:
        return False
    except HTTPError:
        return False
    return True


@pytest.fixture(scope="session")
def ensure_broker_service(docker_services: Fixture) -> bool:
    """Ensure that broker service is up and responsive."""
    docker_services.wait_until_responsive(
        timeout=300, pause=30, check=lambda: is_broker_responsive()
    )
    return True


@pytest.fixture(scope="session")
def ensure_connect_service(docker_services: Fixture) -> bool:
    """Ensure that connect service is up and responsive."""
    docker_services.wait_until_responsive(
        timeout=300, pause=30, check=lambda: is_connect_responsive()
    )
    return True


@pytest.mark.docker
def test_integration_broker_connect(
    ensure_broker_service: Fixture, ensure_connect_service: Fixture
) -> None:
    """Test kafkaconnect with a Kafka broker and Kafka Connect.

    pytest-docker uses the docker-compose.yaml in the test directory.
    """
    broker_url = Config.broker_url
    admin_client = AdminClient({"bootstrap.servers": broker_url})
    t1 = NewTopic(topic="test.t1", num_partitions=1)
    t2 = NewTopic(topic="test.t2", num_partitions=1)
    t3 = NewTopic(topic="test.t3", num_partitions=1)
    # Create test topics in Kafka
    try:
        admin_client.create_topics([t1, t2, t3])
        time.sleep(1)
    except KafkaException:
        return None
    # Test topic discovery
    topic = Topic(
        broker_url=broker_url, topic_regex="test.*", excluded_topics="test.t1"
    )
    assert "test.t2" in topic.names
    assert "test.t3" in topic.names
    # Configure the connector
    connect = Connect(connect_url=Config.connect_url)
    connect_config = InfluxConfig(
        name="influxdb-sink",
        connect_influx_url="http://localhost:8086",
        connect_influx_db="mydb",
        tasks_max=1,
        connect_influx_username="foo",
        connect_influx_password="bar",
        connect_influx_error_policy="foo",
        connect_influx_max_retries="1",
        connect_influx_retry_interval="1",
        connect_progress_enabled=True,
    )
    connect_config.update_topics(topic.names)
    # Create the connector using the Kafka Connect API
    connect.create_or_update(
        name="influxdb-sink", connect_config=connect_config.asjson()
    )
    # List connectors from the Kafka Connect API
    list = connect.list()
    assert "influxdb-sink" in list
